/**
   This is an interface for a collection of objects that are arranged in
   a ring. One object is marked as the current one, and you can advance 
   this marker from one object to the next one.
   
   @author Charles Hoot
   @author Frank M. Carrano
   @version 5.0
 */
public interface RingInterface<T>
{
	/** Adds an item to the end of this ring.
		 @param toAdd  The item to add. */
	public void add(T toAdd);
	
	/** Removes and returns the entry at the beginning of this ring.
		 @return  The first entry in the ring, or null if the ring is empty. */
	public T remove();
	
	/** Removes all of the entries from this ring. */
	public void clear();
	
	/** Returns the current entry in this ring.
		 @return  The current entry in the ring, or null if the ring is empty. */
	public T getCurrentEntry();
	
	/** Advances the marker of the current entry by one entry in this ring. */
	public void advance();
	
	/** Gets the number of entries in this ring.
		 @return  The number of entries in the ring. */
	public int getLength();
	
	/** Sees whether this ring is empty. 
		 @return  True if there are no items in the ring, false otherwise. */
	public boolean isEmpty();
} // end RingInterface
